        title '400 baud BPSK demodulator'
; modified for leonid monitor by 
;	R.Suban 9H1EJ (9h1ej@amsat.org)
; Original Author: 
;         Ulrich Bangert, DF6JB 
;         Ortholzer Weg 1        
;         D-27243 Gross Ippener
;         Email: df6jb@csi.com   
;    
	opt	mu
	opt	cex,mex

; ==============================================================
; Program Equates
;
SampleFreq equ 9600.0
LInputGain equ 5.0		;Gain can be from 0 - 15
RInputGain equ 0.0
OutputAtten equ 15.0		;Attenuation can be from 0 - 15
Radio		equ	1		; Radio 1 or 2
TuneRadio	equ	1		; 1 use up/dwn to tune radio
					; 0 adjust reference carrier
buflen	equ	4		;I/O buffer length

tunewin equ	    0.05
newbit  equ     0
syntest equ     1
scanbit equ     2
carrier equ     3
length  equ     512

	nolist
	include 'leonid'
	list
; ==============================================================
; Macro Definitions 
;
; macro for heartbeat  LED handling
copled	macro	mode
	b\mode	#14,x:$ffe4
	endm

; macro for block  LED handling
catled	macro	mode
	b\mode	#3,x:$ffe4
	endm

; macro for carrier detect LED handling
dcdled	macro	mode
	b\mode	#13,x:$ffe4
	endm

; macro for UP handling port 1
up1led	macro	mode
	b\mode	#7,x:$ffe4
	endm

; macro for DOWN handling port 1
dwn1led macro	mode
	b\mode	#8,x:$ffe4
	endm

; macro for UP handling port 2
up2led	macro	mode
	b\mode	#5,x:$ffe4
	endm

; macro for DOWN handling port 2
dwn2led macro	mode
	b\mode	#6,x:$ffe4
	endm

; macro for immediate move
movi	macro	data,dest
	move	data,a1
	move	a1,dest
	endm

; macro for fir filters, Input x0, Output in a
fir     macro   taps,states,coefs
        jmp     _fstart
_sstore dc      0
_cstore dc      0
_fstart move    #taps-1,m0                      ;mod(ntaps)
        move    #taps-1,m4                      ;mod(ntaps)
        movem   p:_sstore,a
        tst     a
        jne     _dof
        move    #states,r0
        movem   r0,p:_sstore
        move    #coefs,r4
        movem   r4,p:_cstore
_dof    movem   p:_sstore,r0                    ;point to filter states
        move    p:_cstore,r4                    ;point to filter coefficients
        nop
        clr     a    x0,x:(r0)+  y:(r4)+,y0     ;save first state
        rep     #taps-1
        mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
        macr    x0,y0,a  (r0)-
        movem   r0,p:_sstore
        movem   r4,p:_cstore
        endm


; macro for Hilbert transformators, Input x0, i-Output in b, q-Output in a
hilbert macro   taps,states,coefs
        jmp     _hstart
_sstore dc      0
_cstore dc      0
_hstart move    #taps-1,m0                      ;mod(ntaps)
        move    #taps-1,m4                      ;mod(ntaps)
        movem   p:_sstore,a
        tst     a
        jne     _doh
        move    #states,r0
        movem   r0,p:_sstore
        move    #coefs,r4
        movem   r4,p:_cstore
_doh    movem   p:_sstore,r0                    ;point to filter states
        move    p:_cstore,r4                    ;point to filter coefficients
        nop
        clr     a    x0,x:(r0)+  y:(r4)+,y0     ;save first state
        rep     #((taps-1)/2)                   ;do first half of filtering
        mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
        move    x0,b                            ;store delayed input in b
        rep     #((taps-1)/2)                   ;do second half of filtering
        mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
        macr    x0,y0,a  (r0)-
        movem   r0,p:_sstore
        movem   r4,p:_cstore
        endm

; macro for generating a precision sine wave
; theta must be declared long and step as two words in y memory
; sets #whichbit,y:sinflags when a 2 pi overflow occurs
presin  macro   step,theta,where,whichbit
        jmp     <_doit
_store  dc      0
_doit   move    #$ffff,m0               ;init pointer registers
        move    #$100,r0
        clr     a
        clr     b
        move    l:theta,a10             ;move current phase long to a
        move    y:step+1,b0             ;move low word step to b0
        asl     b                       ;get rid of it's sign bit
        move    y:step,b1               ;move hi word step to b1
        add     b,a                     ;compute next phase
        move    #>$FF,x1
        and     x1,a1                   ;limit phase to 360 degrees
        move    a10,l:theta             ;store new phase value
        move    a1,n0
        lsl     a
        asr     a
        move    a0,y0
        nop
        move    y:(r0+n0),b             ;get first result for n0
        move    b,y:where               ;intermediate store
        move    #>1,x0
        add     x0,a
        and     x1,a1
        move    a1,n0
        nop
        move    y:(r0+n0),b             ;get second result for n0+1
        move    y:where,x1              ;now compute a better value
        sub     x1,b
        move    b,x1
        macr    x1,y0,b
        move    y:where,a
        add     b,a
        move    a,y:where               ;and store the end result
        jge     _ok
        move    p:_store,b
        move    #0,x0
        cmp     x0,b
        jmi     _ok
        bset    #whichbit,y:sinflags
_ok     move    a,p:_store
        endm

; macro for generating a precision cos wave
; theta must be declared long and step as two words in y memory
; may only be used in conjunction (immediatly after) precsin
; sets #whichbit,y:cosflags when a 2 pi overflow occurs
precos  macro   step,theta,where,whichbit
        jmp     <_doit
_store  dc      0
_doit   move    #$ffff,m0               ;init pointer registers
        move    #$100,r0
        clr     a
        clr     b
        move    l:theta,a10
        move    #>$40,b
        add     b,a
        move    #>$100,y1
        move    #>$FF,x1
        and     x1,a1
        move    a1,b1
        move    a0,b0
        move    a1,n0
        lsl     a
        asr     a
        move    a0,y0
        move    y:(r0+n0),b
        move    b,y:where
        move    #>1,x0
        add     x0,a
        and     x1,a1
        move    a1,n0
        nop
        move    y:(r0+n0),b
        move    y:where,x1
        sub     x1,b
        move    b,x1
        macr    x1,y0,b
        move    y:where,a
        add     b,a
        move    a,y:where
        jge     _ok
        move    p:_store,b
        move    #0,x0
        cmp     x0,b
        jmi     _ok
        bset    #whichbit,y:cosflags
_ok     move    a,p:_store
        endm

; macro for generating a precision sin wave with a phase of +180 deg
; theta must be declared long and step as two words in y memory
; may only be used in conjunction (immediatly after) precsin
; sets #whichbit,y:sinflags when a 2 pi overflow occurs
psin45  macro   step,theta,where,whichbit
        jmp     <_doit
_store  dc      0
_doit   move    #$ffff,m0               ;init pointer registers
        move    #$100,r0
        clr     a
        clr     b
        move    l:theta,a10
        move    #>$80,b
        add     b,a
        move    #>$100,y1
        move    #>$FF,x1
        and     x1,a1
        move    a1,b1
        move    a0,b0
        move    a1,n0
        lsl     a
        asr     a
        move    a0,y0
        move    y:(r0+n0),b
        move    b,y:where
        move    #>1,x0
        add     x0,a
        and     x1,a1
        move    a1,n0
        nop
        move    y:(r0+n0),b
        move    y:where,x1
        sub     x1,b
        move    b,x1
        macr    x1,y0,b
        move    y:where,a
        add     b,a
        move    a,y:where
        jge     _ok
        move    p:_store,b
        move    #0,x0
        cmp     x0,b
        jmi     _ok
        bset    #whichbit,y:sinflags
_ok     move    a,p:_store
        endm

; macro for calculating the square of a value
; input in reg output in reg
sqr     macro  reg
        move   reg,x0
        mpy    x0,x0,reg
        endm

; macro for calculating the square root
; Input in a, Output in a
sqrt    macro
        move   a1,y1
        move   a0,y0
        clr    b                          ; init root and guess
        move   #$400000,x1                ; init bit to test
        move   #$400000,x0
        do     #23,_endrt
        mpy    -x0,x0,a                   ; square and negate the guess
        add    y,a                        ; compare double precision input
        tge    x0,b                       ; update root if input >= guess
        tfr    x1,a                       ; get bit to test
        asr    a                          ; shift to next bit to test
        add    b,a  a,x1                  ; form new guess
        move   a,x0                       ; save new guess
_endrt  move   b,a
        endm

; macro to compute 2^(-16)/a, works only with positive inputs
; input in a output in a
invert  macro
        move  a,x0
        and   #$fe,CCR
        move  #>$0000FF,a
        rep   #24
        div   x0,a
        add   x0,a
        move  a0,x0
        move  x0,a
        endm

; macro for a first order high-pass IIR filter, input in a, output in a
; frq is -3dB point (fc/fs)
hpass	macro	acc,frq,store
c1	set	-1.0/@cos(6.28319*frq)*(1.0-@sin(6.28319*frq))
	move		    acc,x0
	move		    #>(1.0-c1)/2.0,y0
	mpyr	x0,y0,acc   y:store,y0
	move		    acc,y:store
	sub	y0,acc	    y:store+1,x0
	move		    #>c1,y0
	macr	-x0,y0,acc
	move		    acc,y:store+1
	endm

; macro for first order low pass IIR filter, input in acc, output in acc
lpass   macro   acc,value
        jmp     <_mstart
_store  dc      0
_mstart move    #value,x1
	move	acc,x0
	mpy	x0,x1,acc   #(1.0-value),x0
	move		    p:_store,x1
	macr	x0,x1,acc
	move		    acc,p:_store
        endm

; ==============================================================
; Program
;
	org	p:user_code

	move		    #buffer+2,r7
	move		    #buflen*4-1,m7

	move		    #buffer,r2
	move		    #4-1,n2
	move		    #buflen*4-1,m2

	move    #4,omr                 ;operating mode 4 (sintable enabled)

	move	#>@cvi(SampleFreq),x0
	move	x0,x:hbeat

	ctrlcd	1,r2,buflen,MIC,LInputGain,RInputGain,LINEO|HEADP,OutputAtten,OutputAtten
	opencd	SampleFreq/1000.0,HPF	;start taking samples at given rate
	clr     a
	clr     b
      opensc

	jsr     startscan               ;start scanning operation

main
	clr	a
	move	x:hbeat,a0
	dec	a
	move	a0,x:hbeat
	jne	<main1
	copled chg
	move	#>@cvi(SampleFreq),x0
	move	x0,x:hbeat

main1	waitblk r2,buflen,1		; wait for one complete block

	; read input
	move	x:(r2)+,a			;left ch
	if	(Radio==1)
	move	x:(r2),b			;right ch
	else
	move	x:(r2),a			;right ch
	endif

	move    a,y:inple

	move    #0,a

	; output sample
	move		    a,y:(r2)+ 		;left channel out
	move		    a,y:(r2)+ 		;right channel out
	move			(r2)+

DoIt    move    y:inple,x0
        hilbert hil1t,hil1s,hil1c       ;produce a qphase signal to the
        move    b,y:iin                 ;and store i/q in iin/qin

        sqr     a                       ;square q
        sqr     b                       ;square i
        add     b,a                     ;sum the squares
        sqrt                            ;take the sqareroot of sum
        invert                          ;compute 2^(-16)/result
        move    a,x0                    ;and...........

        move    y:iin,x1                ;use this as normalizing faktor
        mpy     x0,x1,a                 ;to i channel
        rep     #15
        asl     a
        move    a,y:iin                 ;now we have a level independend
                                        ;signal in iin
        presin  step16,theta16,sin1600,22  ;generate 1600 Hz sine
        precos  step16,theta16,cos1600,22  ;generate 1600 Hz cosine

        move    y:iin,x0                ;mix input with 1600 hz cos
        move    y:cos1600,y0
        mpy     x0,y0,a
        lpass   a,0.5                   ;and low pass filter it
        move    a,y:iinchan             ;store in iinchan

        move    y:iin,x0
        move    y:sin1600,y0            ;mix input with 1600 hz cos
        mpy     x0,y0,a
        move    a,y:source2             ;put unfiltered result in source2
        lpass   a,0.5                   ;low pass filter it
        move    a,y:qinchan             ;and store it to qinchan

        move    y:qinchan,x0            ;multiply qinchan
        move    y:iinchan,y0            ;with iinchan (i.e.:a costas loop)
        mpy     x0,y0,a
        lpass   a,0.5                   ;and low pass filter it
        move    a,y:perr                ;the result is the phase error
                                        ;between input and nco

        move    a,x0                    ;here starts the carrier recover
        move    y:p1gain,y0             ;pll. Not very clever programmed,
        mpy     x0,y0,a                 ;however, it works!
        move    a,x0
        move    #>255,y0
        mpy     x0,y0,a

        move    y:freq16+1,b0
        asl     b
        move    y:freq16,b1
        add     a,b

        move    b1,y:step16
        asr     b
        move    b0,y:step16+1

        move    y:source2,x0            ;now we filter the downmixed input
        fir     lp50t,lp50s,lp50c       ;signal with fir low pass filter
        move    a,y:source2             ;and store it in source2

        move    y:qinchan,a             ;this is do decide whether we switch
        abs     a                       ;the carrier detect le on or off
        lpass   a,0.001

        move    y:iinchan,b
        abs     b
        lpass   b,0.001

        sub     b,a
        lpass   a,0.001
        move    #0.05,b
        sub     b,a
        jpl     cledon
	  dcdled  clr
        bclr    #carrier,y:flags
        btst    #scanbit,y:flags
        jscs    doscan
        jmp     <ledok
cledon
	  dcdled  set
	  bset    #carrier,y:flags
        bclr    #scanbit,y:flags
        jsr     dofollow

ledok   presin  step40,theta40,sin400,0 ;generate 400 Hz sine
        precos  step40,theta40,cos400,0 ;generate 400 Hz cosine
        psin45  step40,theta40,sin45,1  ;generate 400 Hz sine +180 degrees

        move    y:source2,x0            ;what follows looks the same as
        move    y:cos400,y0             ;what we have above, and in fact
        mpy     x0,y0,a                 ;it is: a costas loop for 400 hz.
        lpass   a,0.5
        move    a,y:i2

        move    y:source2,x0
        move    y:sin400,y0
        mpy     x0,y0,a
        lpass   a,0.5
        move    a,y:q2

        move    y:i2,x0
        move    y:q2,y0
        mpy     x0,y0,a
        lpass   a,0.5
        move    a,y:perr2             ;the phase error of the 400 hz loop

        move    a,x0
        move    y:p2gain,y0
        mpy     x0,y0,a
        move    a,x0
        move    #>255,y0
        mpy     x0,y0,a


        move    y:freq40+1,b0
        asl     b
        move    y:freq40,b1
        add     a,b

        move    b1,y:step40
        asr     b
        move    b0,y:step40+1         ;now we have plled to the signal

        move    y:q2,a
        asl     a
        move    a,y:q2

        bclr    #1,y:sinflags         ;test if we are at the end of 
        jcc     filter1               ;integration time 1.
        movi    y:pstore1,y:phase1    ;if yes, store result of integrator
filter1 move    y:q2,x0               ;else filter the sample
        fir     lp4t,lp4s,lp4c
        move    a,y:pstore1


        bclr    #0,y:sinflags         ;now the same stuff for integrator 2
        jcc     filter2
        bset    #newbit,y:flags
        movi    y:pstore2,y:phase2
filter2 move    y:q2,x0
        fir     lp4t,lp5s,lp4c
        move    a,y:pstore2

        bclr    #newbit,y:flags       ;if we have no new bit now, we are 
        jcc     main                  ;ready to return to main loop

        move    y:phase1,a            ;now test which of the integrators
        abs     a                     ;holds the bigger magnitude. This
        move    y:phase2,b            ;is the one we need for decoding...
        abs     b
        cmp     b,a
        jge     ph1
        clr     a
        move    y:phcount,a0
        inc     a
        move    a0,y:phcount
        clr     a
        move    y:phcount,a
        move    #>64,x0
        cmp     x0,a
        jle     <ph2
        move    #>64,a0
        move    a0,y:phcount
        jmp     <ph2
ph1     clr     a
        move    y:phcount,a0
        dec     a
        move    a0,y:phcount
        move    #0,x0
        cmp     x0,a
        jge     <ph2
        move    #0,a0
        move    a0,y:phcount

ph2     move    y:phcount,a
        move    #>32,x0
        cmp     x0,a
        jge     take2
take1   move    y:phase1,x0
        jmp     <okt
take2   move    y:phase2,x0
okt     move    y:psymbol,y0
        move    x0,y:psymbol
        mpy     x0,y0,a
        jlt     was1
        move    y:outbyte,a0
        asl     a
        jmp     <was0
was1    move    y:outbyte,a0
        asl     a
        bset    #0,a0
was0    move    a0,y:outbyte
        btst    #syntest,y:flags
        jscc    testit
        clr     a
        move    y:bcount,a0
        dec     a
        move    a0,y:bcount
        jne     main

        clr     a
        move    y:bycount,a0
        dec     a
        move    a0,y:bycount
        jgt     out
        clr     a
        move    a0,y:bycount
        bclr    #syntest,y:flags
        catled  clr
        jmp     main

out     move    #>8,a0
        move    a0,y:bcount
        move    y:outbyte,x0
	  putc
        jmp     main

testit  move    l:compvec,b
        move    l:testvec,a
        asl     a
        abs     a
        btst    #0,y:outbyte
        jcc     shift0
shift1  bset    #0,a0
shift0  move    #>$0000FF,x0
        and     x0,a
        move    a,l:testvec
        cmp     a,b
        jne     tend
ok      bset    #syntest,y:flags
        move    #>9,a0
        move    a0,y:bcount
        move    #>length+1,a0
        move    a0,y:bycount
        catled  set
tend    rts

startscan
        bset    #scanbit,y:flags
        move    #0.0,b0
        asl     b
        move    #>32,b1
        move    b1,y:freq16
        asr     b
        move    b0,y:freq16+1
        rts

doscan  move    y:freq16+1,b0
        asl     b
        move    y:freq16,b1
        clr     a
        move    #>$000800,a0
        add     a,b
        move    b,l:dummy
        move    b1,y:freq16
        asr     b
        move    b0,y:freq16+1
        move    l:dummy,b
        move    #>48,a
        sub     a,b
        jspl    startscan
        rts

dofollow
        move    y:freq16+1,b0
        asl     b
        move    y:freq16,b1
        move    y:step16+1,a0
        asl     a
        move    y:step16,a1
        sub     b,a
 ;       move    a0,y:demod
        jpl     doup
dodown  
	  if (TuneRadio==0)
	  clr     a
        move    #>$000100,a0
        sub     a,b
        move    b1,y:freq16
        asr     b
        move    b0,y:freq16+1
        jmp     <fend
	  else
	  abs     a
	  move    #tunewin,b
	  sub     b,a
	  jpl	    tunedwn
	  jmp	    <tuneoff
	  endif
doup
	  if (TuneRadio==0)
        clr     a
        move    #>$000100,a0
        add     a,b
        move    b1,y:freq16
        asr     b
        move    b0,y:freq16+1
        jmp     <fend
	  else
	  move    #tunewin,b
	  sub     b,a
	  jpl	    tuneup
	  jmp	    <tuneoff
	  endif

tuneup
	  if (Radio==1)
	  dwn1led clr
	  up1led  set
	  else
	  dwn2led clr
	  up2led  set
	  endif
        jmp     <fend

tunedwn
	  if (Radio==1)
	  up1led   clr
	  dwn1led  set
	  else
	  up2led   clr
	  dwn2led  set
	  endif
        jmp     <fend

tuneoff
	  if (Radio==1)
	  up1led   clr
	  dwn1led  clr
	  else
	  up2led   clr
	  dwn2led  clr
	  endif
fend    rts

; ==============================================================
; Data area definition
;
	org	l:user_data
buffer	dsm	4*buflen	; codec i/o buffer
theta16 dc      @LNG(0,0)
theta40 dc      @LNG(0,0)
compvec dc      @LNG($000039,$15ED30)
testvec dc      @LNG(0,0)
dummy   dc      0

        org     y:*
sinflags dc      0
cosflags dc      0
sin1600 dc      0
sin400  dc      0
cos1600 dc      0
cos400  dc      0
sin45   dc      0
step16  dc      40,0.0                                ;42,0.6666666667
step40  dc      10,0.6666666667
freq16  dc      40,0.0                                ;42,0.6666666667
freq40  dc      10,0.6666666667
inple   dc      0
inpri   dc      0
iin     dc      0
iinchan dc      0
qinchan dc      0
i2      dc      0
q2      dc      0
perr    dc      0
perr2   dc      0
source2 dc      0
demod   dc      0
p1gain  dc      2*100.0/9600.0
p2gain  dc      8*1.0/9600.0
phase1  dc      0
phase2  dc      0
pstore1 dc      0
pstore2 dc      0
flags   dc      0
bcount  dc      8
symbol  dc      0
psymbol dc      0
outbyte dc      0
phcount dc      0
bycount dc      0

        org     y:$1000
lp50c
; FIR lowpass filter
; Generated by FIRfilt(c) 
;  
; DSP56000 family format 
;  
; Edges (Hz): 
;  0...384 
;  903.7037037...4800 
;  
; Specified deviations (dB): 
;  0.4237860143
;  -34.82142857
;  
; Worst deviations (dB): 
;  0.6781332194
;  -30.5870204
;  
; Target precision: fixed point 24 bit, unnormalized 
;  
; 24 coefficients: 

  dc  -2.02722549E-002    ; h[0]
  dc  -1.13016367E-002    ; h[1]
  dc  -9.95731354E-003    ; h[2]
  dc  -4.26924229E-003    ; h[3]
  dc   6.47997856E-003    ; h[4]
  dc   2.23222971E-002    ; h[5]
  dc   4.23884392E-002    ; h[6]
  dc   6.49915934E-002    ; h[7]
  dc   8.77262354E-002    ; h[8]
  dc   1.07979417E-001    ; h[9]
  dc   1.23183131E-001    ; h[10]
  dc   1.31330132E-001    ; h[11]
  dc   1.31330132E-001    ; h[12]
  dc   1.23183131E-001    ; h[13]
  dc   1.07979417E-001    ; h[14]
  dc   8.77262354E-002    ; h[15]
  dc   6.49915934E-002    ; h[16]
  dc   4.23884392E-002    ; h[17]
  dc   2.23222971E-002    ; h[18]
  dc   6.47997856E-003    ; h[19]
  dc  -4.26924229E-003    ; h[20]
  dc  -9.95731354E-003    ; h[21]
  dc  -1.13016367E-002    ; h[22]
  dc  -2.02722549E-002    ; h[23]


lp50t   equ *-lp50c

        org     y:$1100
hil1c
;Coefficients of a 65 staged Hilbert transformator designed with MATLAB
 DC -4.0951116e-007
 DC -3.9476589e-005
 DC -9.5730782e-007
 DC -1.4430708e-004
 DC -1.9906098e-006
 DC -3.9006525e-004
 DC -3.5367140e-006
 DC -8.8363036e-004
 DC -5.6416991e-006
 DC -1.7794399e-003
 DC -8.2543717e-006
 DC -3.2867974e-003
 DC -1.1231090e-005
 DC -5.6785941e-003
 DC -1.4324996e-005
 DC -9.3045193e-003
 DC -1.7192464e-005
 DC -1.4618166e-002
 DC -1.9447481e-005
 DC -2.2239420e-002
 DC -2.0699726e-005
 DC -3.3103521e-002
 DC -2.0631420e-005
 DC -4.8835967e-002
 DC -1.9072441e-005
 DC -7.2804132e-002
 DC -1.5983880e-005
 DC -1.1372992e-001
 DC -1.1525443e-005
 DC -2.0378314e-001
 DC -6.0394836e-006
 DC -6.3376613e-001
 DC  0.0000000e+000
 DC  6.3376613e-001
 DC  6.0394836e-006
 DC  2.0378314e-001
 DC  1.1525443e-005
 DC  1.1372992e-001
 DC  1.5983880e-005
 DC  7.2804132e-002
 DC  1.9072441e-005
 DC  4.8835967e-002
 DC  2.0631420e-005
 DC  3.3103521e-002
 DC  2.0699726e-005
 DC  2.2239420e-002
 DC  1.9447481e-005
 DC  1.4618166e-002
 DC  1.7192464e-005
 DC  9.3045193e-003
 DC  1.4324996e-005
 DC  5.6785941e-003
 DC  1.1231090e-005
 DC  3.2867974e-003
 DC  8.2543717e-006
 DC  1.7794399e-003
 DC  5.6416991e-006
 DC  8.8363036e-004
 DC  3.5367140e-006
 DC  3.9006525e-004
 DC  1.9906098e-006
 DC  1.4430708e-004
 DC  9.5730782e-007
 DC  3.9476589e-005
 DC  4.0951116e-007
hil1t   equ *-hil1c

        org     y:$1200
lp4c
 ; boxcar integrator
 ;
    dc      0.0434  ;       1
    dc      0.0434  ;       2
    dc      0.0434  ;       3
    dc      0.0434  ;       4
    dc      0.0434  ;       5
    dc      0.0434  ;       6
    dc      0.0434  ;       7
    dc      0.0434  ;       8
    dc      0.0434  ;       9
    dc      0.0434  ;      10
    dc      0.0434  ;      11
    dc      0.0434  ;      12
    dc      0.0434  ;      13
    dc      0.0434  ;      14
    dc      0.0434  ;      15
    dc      0.0434  ;      16
    dc      0.0434  ;      17
    dc      0.0434  ;      18
    dc      0.0434  ;      19
    dc      0.0434  ;      20
    dc      0.0434  ;      21
    dc      0.0434  ;      22
    dc      0.0434  ;      23
    dc      0.0434  ;      24
lp4t    equ  *-lp4c
 
       org     x:$1000
lp50s   dsm lp50t
hil1s   dsm hil1t
lp4s    dsm lp4t
lp5s    dsm lp4t
hbeat	  dc  0

	end

