        title 'Send BPSK 400 baud frames'
; modified for leonid monitor by R.Suban 9H1EJ (9h1ej@amsat.org)
; Original Author: 
;         Ulrich Bangert, DF6JB 
;         Ortholzer Weg 1        
;         D-27243 Gross Ippener
;         Email: df6jb@csi.com   
;    
; Revision information
; Modification history

	opt	mu
	opt	cex,mex

; ==============================================================
; Program Equates
;
SampleFreq equ 9600.0		;can be 8000, 9600, 16000, 32000, 48000
LInputGain equ 0.0		;Gain can be from 0 - 15
RInputGain equ 0.0
OutputAtten equ 0.0		;Attenuation can be from 0 - 15

buflen	equ	4		;I/O buffer length

START		EQU   $100
STEP    	EQU   42.666666667
RELOAD1 	EQU   12
RELOAD2 	EQU   24
RELOAD3 	EQU   24
NTAPS1  	EQU   257

; ==============================================================
; Macro Definitions 
;
	nolist
	include 'leonid'
	list
; ==============================================================
; Program
;
	org	p:user_code

	move	#buffer+2,r7
	move	#buflen*4-1,m7

	move	#buffer,r2
	move	#4-1,n2
	move	#buflen*4-1,m2
	move	#4,omr               ;operating mode 4 (sintable enabled)

INIT
	MOVE    #$FF,X0         ;INITIAL STATE OF NOISE GENERATOR
	MOVE    X0,Y:NSTATE
	MOVE    #DATBLOK,r1     ;SETUP DATBLOK POINTER	
	NOP
	MOVE    y:(r1),a1       ;GET FIRST CHAR
	MOVE    a1,y:CURBYTE    ;AND STORE IT IN VAR CURBYTE
	move    #STATES1,a0
	move    a0,y:TEMPF12    ;point to filter states
      move    #NTAPS1-1,a0
      move    a0,y:TEMPF11    ;mod(ntaps)
      move    #COEF1,a0
      move    a0,y:TEMPF14    ;point to filter COEFficients
      move    #NTAPS1-1,a0
      move    a0,y:TEMPF13    ;mod(ntaps)

	ctrlcd	1,r2,buflen,MIC,LInputGain,RInputGain,LINEO|HEADP,OutputAtten,OutputAtten
	opencd	SampleFreq/1000.0,HPF		;start taking samples at given rate

loop	waitblk r2,buflen,1			; wait for one complete block

	; read input
	move	x:(r2)+,a			;left ch
	move	x:(r2),x0			;right ch

	; output sample
      move    y:SINE,a
      move    y:OUTPUT,b
	move		    a,y:(r2)+ 		;left channel out
	move		    b,y:(r2)+ 		;right channel out
	move			(r2)+

	JSR     DO400           ;GENERATE CURRENT OUTPUT BIT
	JSR     DO800           ;GENERATE BITRATE CLOCK
	JSR     NOISE           ;GENERATE NOISE SAMPLE
	JSR     PRECSIN         ;GENERATE PHASE MODULATED PRECISION SINE
	JSR     DOADD           ;ADD THE RESULTS OF BOTH
	JSR     DOFILT          ;FILTER IT BY A TYPICAL SSB CHANNEL

	jmp	<loop

DO800
	clr     a
      move    y:COUNT1,a0
      dec     a
      jne     END800
      bchg    #0,y:BITCL
      clr     a
      jsr     MODULATE
      clr     a
      move    #<RELOAD1,a0
END800   move    a0,y:COUNT1
      rts


DO400
      clr     a
      move    y:COUNT2,a0
      dec     a
      move    a0,y:COUNT2
      jne     END400
      clr     a
      move    #<RELOAD2,a0
      move    a0,y:COUNT2
      move    y:CURBYTE,a1
      lsl     a
      move    a1,y:CURBYTE
      jcc     SEND0
SEND1 bset    #0,y:DATABIT
      bchg    #0,y:SENDBIT
      jmp     TESTB
SEND0 bclr    #0,y:DATABIT
TESTB clr     a
      move    y:COUNT3,a0
      dec     a
      move    a0,y:COUNT3
      jne     END400
      clr     a
      move    #<RELOAD3,a0
      move    a0,y:COUNT3
      clr     a  (r1)+
      move    y:(r1),a1
      move    a1,y:CURBYTE
      tst     a
      jne     END400
      move    #DATBLOK,r1
      nop
      move    y:(r1),a1
      move    a1,y:CURBYTE
END400 rts

MODULATE
      clr     a
      move    y:SENDBIT,a1
      move    y:BITCL,x0
      eor     x0,a
      btst    #0,a1
      jcc     MODMINUS
MODPLUS  move    y:PSKLEV,a
      abs     a
      move    a,y:PSKLEV
      bset    #0,y:RESBIT
      jmp     MODEND
MODMINUS move    y:PSKLEV,a
      abs     a
      neg     a
      move    a,y:PSKLEV
MODEND   rts

DOADD
      move    y:SINE,a
      move    y:NOIVAL,b
      add     b,a
      move    a,y:OUTPUT
      rts

NOISE                                    ;DO NOISE GENERATION LOOP
      CLR     B   Y:NSTATE,A          ;GET A ZERO IN B
      TST     A   #>$000001,X0        ;CHECK A, GET A 1 IN LSB POSITION
      TNR     X0,B                    ;IF NORMALIZED, THEN XOR=1
      ADD     B,A                     ;SET LSB
      ASL     A                       ;SHIFT BITS
      MOVE    A1,Y:NSTATE             ;SAVE STATE
      CLR     A
      MOVE    Y:NSTATE,X0
      MOVE    Y:NOILEV,Y0
      MACR    X0,Y0,A
      MOVE    A,Y:NOIVAL
      RTS

PRECSIN
     move    #$ffff,m6
     move    #$100,r6
     clr     a
     move    y:THETA,a0
     rep     #8
     asl     a
     move    #(STEP-@cvi(STEP)),b0
     asl     b
     nop
     move    #@cvi(STEP),b1
     add     b,a
     move    #>$FF,x1
     and     x1,a1
     move    a1,b1
     move    a0,b0
     move    a1,n6
     lsl     a
     asr     a
     move    a0,y0
     rep     #8
     asr     b
     move    b0,y:THETA
     btst    #0,y:RESBIT
     jcc     NORESET
     clr     a
     move    a0,y:THETA
     move    a0,n6
NORESET bclr    #0,y:RESBIT
     move    y:(r6+n6),b
     move    b,y:SINE
     move    #>1,x0
     add     x0,a
     and     x1,a1
     move    a1,n6
     nop
     move    y:(r6+n6),b
     move    y:SINE,x1
     sub     x1,b
     move    b,x1
     macr    x1,y0,b
     move    y:SINE,a
     add     b,a
     move    a,y:SINE
     clr     a
     move    y:THETA,a0
     rep     #8
     asl     a
     move    #>$40,b
     add     b,a
     move    #>$FF,x1
     and     x1,a1
     move    a1,b1
     move    a0,b0
     move    a1,n6
     lsl     a
     asr     a
     move    a0,y0
     move    y:(r6+n6),b
     move    b,y:COSINE
     move    #>1,x0
     add     x0,a
     and     x1,a1
     move    a1,n6
     nop
     move    y:(r6+n6),b
     move    y:COSINE,x1
     sub     x1,b
     move    b,x1
     macr    x1,y0,b
     move    y:COSINE,a
     add     b,a
     move    a,y:COSINE
     clr     a
     move    y:SINE,x0
     move    y:PSKLEV,y0
     macr    x0,y0,a
     move    a,y:SINE
     clr     a
     move    y:COSINE,x0
     move    y:PSKLEV,y0
     macr    x0,y0,a
     move    a,y:COSINE
     rts

DOFILT
     move    y:OUTPUT,x0
     move    y:TEMPF11,m0
     move    y:TEMPF12,r0
     move    y:TEMPF13,m4
     move    y:TEMPF14,r4
     nop
     clr     a    x0,x:(r0)+  y:(r4)+,y0     ;save first state
     do      #NTAPS1-1,ENDFILT
     mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
ENDFILT
     macr    x0,y0,a  (r0)-
     move    a,y:OUTPUT                      ;store result
     move    m0,y:TEMPF11
     move    r0,y:TEMPF12
     move    m4,y:TEMPF13
     move    r4,y:TEMPF14
     rts
	
; ==============================================================
; Data area definition
;

	org	l:user_data
buffer	dsm	4*buflen	; codec i/o buffer
LastL	equ	*

      org     Y:$1000

NOILEV  DC      0.0
PSKLEV  DC      0.3
NSTATE  DS      1
NOIVAL  DC      0
THETA   DC      0
SINE    DC      0
COSINE  DC      0
OUTPUT  DC      0
COUNT1  DC      RELOAD1
COUNT2  DC      RELOAD2
COUNT3  DC      RELOAD3
CURBYTE DC      0
BITCL   DC      0
DATABIT DC      0
SENDBIT DC      0
RESBIT  DC      0
TEMPF11 DC      0
TEMPF12 DC      0
TEMPF13 DC      0
TEMPF14 DC      0
DATBLOK DCB     $39,$15,$ED,$30,13,10
        DUP     8
        DC      "The quick brown fox jumps over the lazy dogs back 1234567890"
        DCB     32,13,10
        ENDM
        DC      "DF6JB "
        DUP     50
        DCB     $50,$50,$50
        ENDM
TEXTEND DC      0

        org     Y:$2000

COEF1
 ; Coefficients computed by FIRDES, Copyright Ulrich Bangert, DF6JB
 ; Date: 20.07.1996 Time: 16:35:37
 ; Windowfunction: Blackman
 ; Sampling frequency: 9600 Hz
 ;
 ; Highpass at 450 Hz
 ; Lowpass at  2750 Hz
 ;
    dc     -0.000000028965  ;       1
    dc      0.000000295583  ;       2
    dc      0.000000945039  ;       3
    dc     -0.000000274073  ;       4
    dc      0.000002228156  ;       5
    dc      0.000008436386  ;       6
    dc      0.000003945384  ;       7
    dc      0.000000263591  ;       8
    dc      0.000014805443  ;       9
    dc      0.000013229528  ;      10
    dc     -0.000012310933  ;      11
    dc     -0.000003900895  ;      12
    dc      0.000014505775  ;      13
    dc     -0.000027804924  ;      14
    dc     -0.000054840902  ;      15
    dc     -0.000008683570  ;      16
    dc     -0.000021995773  ;      17
    dc     -0.000096181602  ;      18
    dc     -0.000051171805  ;      19
    dc      0.000022692472  ;      20
    dc     -0.000055397165  ;      21
    dc     -0.000073743973  ;      22
    dc      0.000085407543  ;      23
    dc      0.000094385975  ;      24
    dc     -0.000028664476  ;      25
    dc      0.000103724120  ;      26
    dc      0.000263489110  ;      27
    dc      0.000085820333  ;      28
    dc      0.000021042878  ;      29
    dc      0.000272105382  ;      30
    dc      0.000191332340  ;      31
    dc     -0.000142513411  ;      32
    dc     -0.000000000000  ;      33
    dc      0.000167558318  ;      34
    dc     -0.000264545049  ;      35
    dc     -0.000442621035  ;      36
    dc     -0.000040295886  ;      37
    dc     -0.000193633895  ;      38
    dc     -0.000701241370  ;      39
    dc     -0.000326051367  ;      40
    dc      0.000106600067  ;      41
    dc     -0.000416066128  ;      42
    dc     -0.000447268195  ;      43
    dc      0.000459989085  ;      44
    dc      0.000412825013  ;      45
    dc     -0.000202733402  ;      46
    dc      0.000550268717  ;      47
    dc      0.001250649719  ;      48
    dc      0.000347687506  ;      49
    dc      0.000167889680  ;      50
    dc      0.001306199642  ;      51
    dc      0.000822686124  ;      52
    dc     -0.000538435105  ;      53
    dc      0.000183786244  ;      54
    dc      0.000746610492  ;      55
    dc     -0.001050575292  ;      56
    dc     -0.001572456889  ;      57
    dc     -0.000038458405  ;      58
    dc     -0.000818768967  ;      59
    dc     -0.002609637883  ;      60
    dc     -0.001097642115  ;      61
    dc      0.000237688559  ;      62
    dc     -0.001708989084  ;      63
    dc     -0.001578045659  ;      64
    dc      0.001477656122  ;      65
    dc      0.001042591674  ;      66
    dc     -0.000835567259  ;      67
    dc      0.001859212933  ;      68
    dc      0.003858134996  ;      69
    dc      0.000905379715  ;      70
    dc      0.000740584291  ;      71
    dc      0.004266375534  ;      72
    dc      0.002451955848  ;      73
    dc     -0.001406312984  ;      74
    dc      0.001089791623  ;      75
    dc      0.002380358800  ;      76
    dc     -0.003043249790  ;      77
    dc     -0.004081134711  ;      78
    dc      0.000220399904  ;      79
    dc     -0.002576214206  ;      80
    dc     -0.007392875743  ;      81
    dc     -0.002824848891  ;      82
    dc      0.000265007684  ;      83
    dc     -0.005381496096  ;      84
    dc     -0.004401460288  ;      85
    dc      0.003776781522  ;      86
    dc      0.001897351507  ;      87
    dc     -0.002687433988  ;      88
    dc      0.005144806887  ;      89
    dc      0.009833055219  ;      90
    dc      0.001875742672  ;      91
    dc      0.002521275423  ;      92
    dc      0.011819666731  ;      93
    dc      0.006270037295  ;      94
    dc     -0.003104125941  ;      95
    dc      0.004269955643  ;      96
    dc      0.006677292132  ;      97
    dc     -0.007875275694  ;      98
    dc     -0.009428511204  ;      99
    dc      0.001451269783  ;     100
    dc     -0.007426058329  ;     101
    dc     -0.019588640500  ;     102
    dc     -0.006828605695  ;     103
    dc     -0.000356931772  ;     104
    dc     -0.016268032373  ;     105
    dc     -0.012138197560  ;     106
    dc      0.009653342566  ;     107
    dc      0.002665178163  ;     108
    dc     -0.008779057611  ;     109
    dc      0.015122869572  ;     110
    dc      0.027152430213  ;     111
    dc      0.003959915575  ;     112
    dc      0.009340134684  ;     113
    dc      0.038428114824  ;     114
    dc      0.019432719456  ;     115
    dc     -0.008274224511  ;     116
    dc      0.019596603292  ;     117
    dc      0.025586473052  ;     118
    dc     -0.029410997361  ;     119
    dc     -0.032688374409  ;     120
    dc      0.010137680468  ;     121
    dc     -0.038154385328  ;     122
    dc     -0.103150031316  ;     123
    dc     -0.036855111512  ;     124
    dc     -0.010346287479  ;     125
    dc     -0.163676468545  ;     126
    dc     -0.158658569680  ;     127
    dc      0.217540869383  ;     128
    dc      0.479166666667  ;     129
    dc      0.217540869383  ;     130
    dc     -0.158658569680  ;     131
    dc     -0.163676468545  ;     132
    dc     -0.010346287479  ;     133
    dc     -0.036855111512  ;     134
    dc     -0.103150031316  ;     135
    dc     -0.038154385328  ;     136
    dc      0.010137680468  ;     137
    dc     -0.032688374409  ;     138
    dc     -0.029410997361  ;     139
    dc      0.025586473052  ;     140
    dc      0.019596603292  ;     141
    dc     -0.008274224511  ;     142
    dc      0.019432719456  ;     143
    dc      0.038428114824  ;     144
    dc      0.009340134684  ;     145
    dc      0.003959915575  ;     146
    dc      0.027152430213  ;     147
    dc      0.015122869572  ;     148
    dc     -0.008779057611  ;     149
    dc      0.002665178163  ;     150
    dc      0.009653342566  ;     151
    dc     -0.012138197560  ;     152
    dc     -0.016268032373  ;     153
    dc     -0.000356931772  ;     154
    dc     -0.006828605695  ;     155
    dc     -0.019588640500  ;     156
    dc     -0.007426058329  ;     157
    dc      0.001451269783  ;     158
    dc     -0.009428511204  ;     159
    dc     -0.007875275694  ;     160
    dc      0.006677292132  ;     161
    dc      0.004269955643  ;     162
    dc     -0.003104125941  ;     163
    dc      0.006270037295  ;     164
    dc      0.011819666731  ;     165
    dc      0.002521275423  ;     166
    dc      0.001875742672  ;     167
    dc      0.009833055219  ;     168
    dc      0.005144806887  ;     169
    dc     -0.002687433988  ;     170
    dc      0.001897351507  ;     171
    dc      0.003776781522  ;     172
    dc     -0.004401460288  ;     173
    dc     -0.005381496096  ;     174
    dc      0.000265007684  ;     175
    dc     -0.002824848891  ;     176
    dc     -0.007392875743  ;     177
    dc     -0.002576214206  ;     178
    dc      0.000220399904  ;     179
    dc     -0.004081134711  ;     180
    dc     -0.003043249790  ;     181
    dc      0.002380358800  ;     182
    dc      0.001089791623  ;     183
    dc     -0.001406312984  ;     184
    dc      0.002451955848  ;     185
    dc      0.004266375534  ;     186
    dc      0.000740584291  ;     187
    dc      0.000905379715  ;     188
    dc      0.003858134996  ;     189
    dc      0.001859212933  ;     190
    dc     -0.000835567259  ;     191
    dc      0.001042591674  ;     192
    dc      0.001477656122  ;     193
    dc     -0.001578045659  ;     194
    dc     -0.001708989084  ;     195
    dc      0.000237688559  ;     196
    dc     -0.001097642115  ;     197
    dc     -0.002609637883  ;     198
    dc     -0.000818768967  ;     199
    dc     -0.000038458405  ;     200
    dc     -0.001572456889  ;     201
    dc     -0.001050575292  ;     202
    dc      0.000746610492  ;     203
    dc      0.000183786244  ;     204
    dc     -0.000538435105  ;     205
    dc      0.000822686124  ;     206
    dc      0.001306199642  ;     207
    dc      0.000167889680  ;     208
    dc      0.000347687506  ;     209
    dc      0.001250649719  ;     210
    dc      0.000550268717  ;     211
    dc     -0.000202733402  ;     212
    dc      0.000412825013  ;     213
    dc      0.000459989085  ;     214
    dc     -0.000447268195  ;     215
    dc     -0.000416066128  ;     216
    dc      0.000106600067  ;     217
    dc     -0.000326051367  ;     218
    dc     -0.000701241370  ;     219
    dc     -0.000193633895  ;     220
    dc     -0.000040295886  ;     221
    dc     -0.000442621035  ;     222
    dc     -0.000264545049  ;     223
    dc      0.000167558318  ;     224
    dc     -0.000000000000  ;     225
    dc     -0.000142513411  ;     226
    dc      0.000191332340  ;     227
    dc      0.000272105382  ;     228
    dc      0.000021042878  ;     229
    dc      0.000085820333  ;     230
    dc      0.000263489110  ;     231
    dc      0.000103724120  ;     232
    dc     -0.000028664476  ;     233
    dc      0.000094385975  ;     234
    dc      0.000085407543  ;     235
    dc     -0.000073743973  ;     236
    dc     -0.000055397165  ;     237
    dc      0.000022692472  ;     238
    dc     -0.000051171805  ;     239
    dc     -0.000096181602  ;     240
    dc     -0.000021995773  ;     241
    dc     -0.000008683570  ;     242
    dc     -0.000054840902  ;     243
    dc     -0.000027804924  ;     244
    dc      0.000014505775  ;     245
    dc     -0.000003900895  ;     246
    dc     -0.000012310933  ;     247
    dc      0.000013229528  ;     248
    dc      0.000014805443  ;     249
    dc      0.000000263591  ;     250
    dc      0.000003945384  ;     251
    dc      0.000008436386  ;     252
    dc      0.000002228156  ;     253
    dc     -0.000000274073  ;     254
    dc      0.000000945039  ;     255
    dc      0.000000295583  ;     256
    dc     -0.000000028965  ;     257


    org     x:$2000

STATES1 dsm     NTAPS1          ;filter states
	end

